#include "tycho.h"
#include "mapprops.h"
#include "menu.h"
#include "project.h"
#include "../ui/ui_menu.h"
#include "../ui/ui_about.h"
#include "../ui/ui_license.h"

#include <QCloseEvent>
#include <QFileDialog>
#include <QMdiSubWindow>
#include <iostream>

Menu::Menu(QWidget *parent) :
   QMainWindow(parent),
   ui(new Ui::Menu)
{
   ui->setupUi(this);

   ui->actionAbout->setShortcut(QKeySequence(QKeySequence::HelpContents));
   ui->actionClose->setShortcut(QKeySequence(QKeySequence::Close));
   ui->actionMapProps->setShortcut(QKeySequence(tr("Ctrl+P")));
   ui->actionNew->setShortcut(QKeySequence(QKeySequence::New));
   ui->actionOpen->setShortcut(QKeySequence(QKeySequence::Open));
   ui->actionQuit->setShortcut(QKeySequence(QKeySequence::Quit));

   dbgPrintFunc();
}

Menu::~Menu()
{
   dbgPrintFunc();
}

void Menu::mapNew()
{
   QScopedPointer view{new ProjectView};

   addProject(view.take());
}

void Menu::mapOpen()
{
   auto fname =
      QFileDialog::getOpenFileName(
         this,
         tr("Open Map File"),
         QString(),
         tr("Marathon Map files (*.scen *.sceA Map)"));

   QScopedPointer view{new ProjectView};

   if(view->model()->open(fname)) {
      addProject(view.take());
   }
}

void Menu::openAbout()
{
   QDialog   dlg{this};
   Ui::About ui{};

   ui.setupUi(&dlg);

   auto text = ui.labelText->text();

   text.replace("AUTHORS",
                tr(tychoAuthors()).replace(':', ", ").toHtmlEscaped());
   text.replace("HOMEPAGE", tr(tychoHomepage()));
   text.replace("REPOSITORY", tr(tychoRepository()));
   text.replace("VERSION", tr(tychoVersion()));

   ui.labelText->setText(text);

   connect(ui.btnLicense, &QPushButton::clicked, this, [&](){
      openLicense(&dlg);
   });

   dlg.exec();
}

void Menu::openLicense(QWidget *parent)
{
   QDialog     dlg{parent};
   Ui::License ui{};

   ui.setupUi(&dlg);

   ui.text->setPlainText(tychoLicenseText());

   connect(ui.btnCopy, &QPushButton::clicked, this, [&]() {
      ui.text->selectAll();
      ui.text->copy();
   });

   dlg.exec();
}

void Menu::openMapProperties()
{
   auto view = activeProject();

   if(view) {
      MapProps props{view->model(), view};
      props.exec();
   }
}

void Menu::updateActions()
{
   auto view   = activeProject();
   bool active = view != nullptr;

   ui->actionClose->setEnabled(active);
   ui->actionMapProps->setEnabled(active);

   dbgPrintFunc();
}

void Menu::closeEvent(QCloseEvent *event)
{
   for(auto *win : ui->mdiArea->subWindowList()) {
      if(!win->close()) {
         event->ignore();
         return;
      }
   }

   event->accept();
}

ProjectView *Menu::activeProject() const
{
   return qobject_cast<ProjectView *>(activeSubWindow());
}

QMdiSubWindow *Menu::activeSubWindow() const
{
   return ui->mdiArea->activeSubWindow();
}

void Menu::addProject(ProjectView *view)
{
   auto win = ui->mdiArea->addSubWindow(view);
   win->showMaximized();
}

// EOF
